from fconv.formats.json import Json
from fconv.formats.toml import Toml
from fconv.formats.xml import Xml
from fconv.formats.yaml import Yaml

SUPPORTED_FORMATS = {"json": Json, "yaml": Yaml, "toml": Toml, "xml": Xml}


def get_supported_formats():
    return [k for k in SUPPORTED_FORMATS.keys()]



from abc import ABC, abstractmethod
from typing import Any, Dict, Optional


class BaseFormat(ABC):
    """Base class for implementing all the format"""

    @abstractmethod
    def load(self, src_ctx: Any) -> Any:
        raise NotImplementedError()

    @abstractmethod
    def dump(self, internal: Any) -> Optional[str]:
        raise NotImplementedError()


class BaseDictionalizeFormat(BaseFormat):
    """Base class for implementing format that can convert to/from dict"""

    def __init__(self, load_data_key: str = "", dump_data_key: str = ""):
        self.load_data_key = load_data_key
        self.dump_data_key = dump_data_key

    def get_load_kwargs(self, src_ctx: str, opt: Dict[str, Any]) -> Dict[str, Any]:
        _opt = opt if opt else {}
        _opt[self.load_data_key] = src_ctx
        return _opt

    def get_dump_kwargs(
        self, internal: Dict[str, Any], opt: Dict[str, Any]
    ) -> Dict[str, Any]:
        _opt = opt if opt else {}
        _opt[self.dump_data_key] = internal
        return _opt



from logging import ERROR, Formatter, StreamHandler, getLogger
from sys import stdout
from typing import Any, Dict


class SingletonType(type):
    _instances: Dict[Any, Any] = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonType, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Logger(object, metaclass=SingletonType):
    _logger = None

    def __init__(self, level=ERROR):
        self._logger = getLogger()
        self._logger.setLevel(level)
        formatter = Formatter(
            "\033[32m %(module)s.py\033[0m %(funcName)s() ln.%(lineno)s | [%(levelname)s] %(message)s"
        )

        streamHandler = StreamHandler(stdout)
        streamHandler.setFormatter(formatter)
        self._logger.addHandler(streamHandler)

    @property
    def logger(self):
        return self._logger


def logg(f):
    def wrapped(*arg, **kwargs):
        log = Logger().logger
        log.debug(f"\033[34m[ENTER]:\033[36m {arg = }")
        log.debug(f"\033[34m[ENTER]:\033[36m {f.__name__ = }")
        log.debug(f"\033[34m[ENTER]:\033[36m {kwargs = }")

        return_value = f(*arg, **kwargs)

        log.debug(f"\033[33m[EXIT]:\033[35m {return_value=}")
        log.debug("========================================")
        return return_value

    return wrapped



import argparse
from argparse import Namespace
from logging import DEBUG, ERROR

from fconv import HELP, __doc__, __prog__, __version__
from fconv.core import Former
from fconv.formats import SUPPORTED_FORMATS, get_supported_formats
from fconv.util import Logger


def _set_opts() -> Namespace:
    p = argparse.ArgumentParser(prog=__prog__, description=__doc__)

    # Required
    p.add_argument(
        "source",
        help=HELP["source"],
        choices=get_supported_formats(),
        metavar="source-format",
    )
    p.add_argument("-i", required=True, help=HELP["infile"])
    p.add_argument(
        "target",
        help=HELP["target"],
        choices=get_supported_formats(),
        metavar="target-format",
    )

    # Optional
    p.add_argument(
        "--v",
        help=HELP["version"],
        action="version",
        version=f"{__prog__}: v{__version__}",
    ),
    p.add_argument("--debug", help=HELP["debug"], action="store_true")

    p.add_argument("-o", help=HELP["outfile"])
    # json input
    p.add_argument(
        "--json-float-as-int", help=HELP["json_float_as_int"], action="store_true"
    )
    p.add_argument(
        "--json-float-as-str", help=HELP["json_float_as_str"], action="store_true"
    )
    p.add_argument(
        "--json-int-as-float", help=HELP["json_int_as_float"], action="store_true"
    )
    p.add_argument(
        "--json-int-as-str", help=HELP["json_int_as_str"], action="store_true"
    )
    # json output
    p.add_argument("--json-skip-keys", help=HELP["json_skip_keys"], action="store_true")
    p.add_argument(
        "--json-ignore-check-circular",
        help=HELP["json_ignore_check_circular"],
        action="store_true",
    )
    p.add_argument(
        "--json-disallow-nan", help=HELP["json_disallow_nan"], action="store_true"
    )
    p.add_argument("--json-indent", help=HELP["json_indent"], type=int)
    p.add_argument("--json-sort-keys", help=HELP["json_sort_keys"], action="store_true")

    # yaml output
    p.add_argument(
        "--yaml-explicit-start", help=HELP["yaml_explicit_start"], action="store_true"
    )
    p.add_argument(
        "--yaml-explicit-end", help=HELP["yaml_explicit_end"], action="store_true"
    )
    p.add_argument("--yaml-indent", help=HELP["yaml_indent"], type=int)

    # xml intput
    p.add_argument(
        "--xml-process-namespaces",
        help=HELP["xml_process_namespaces"],
        action="store_true",
    )
    p.add_argument(
        "--xml-process-comments", help=HELP["xml_process_comments"], action="store_true"
    )

    # xml output
    p.add_argument(
        "--xml-particle-document",
        help=HELP["xml_particle_document"],
        action="store_true",
    )
    p.add_argument(
        "--xml-disable-pretty", help=HELP["xml_disable_pretty"], action="store_true"
    )

    a = p.parse_args()
    return a


def parse_args(a):
    in_opt, out_opt = {}, {}
    if a.json_float_as_int and a.json_float_as_str:
        raise ValueError(
            "It can use either --json-float-as-int or --json-float-as-str "
        )
    if a.json_int_as_str and a.json_int_as_float:
        raise ValueError("It can use either --json-int-as-float or --json-int-as-str ")

    # FIXME: too redundant
    # json input
    if a.json_float_as_int:
        in_opt["parse_float"] = __int
    elif a.json_float_as_str:
        in_opt["parse_float"] = str

    if a.json_int_as_float:
        in_opt["parse_int"] = float
    elif a.json_int_as_str:
        in_opt["parse_int"] = str

    # json output
    if a.json_skip_keys:
        out_opt["skipkeys"] = True

    if a.json_ignore_check_circular:
        out_opt["check_circular"] = False

    if a.json_disallow_nan:
        out_opt["allow_nan"] = False

    if a.json_indent is not None:
        out_opt["indent"] = a.json_indent  # type:ignore

    if a.json_sort_keys:
        out_opt["sort_keys"] = a.json_sort_keys

    # yaml output
    if a.yaml_explicit_start:
        out_opt["explicit_start"] = True
    if a.yaml_explicit_end:
        out_opt["explicit_end"] = True
    if a.yaml_indent is not None:
        out_opt["indent"] = a.yaml_indent
    # xml input
    if a.xml_process_namespaces:
        in_opt["process_namespaces"] = True  # type:ignore
    if a.xml_process_comments:
        in_opt["process_comments"] = True  # type:ignore
    # xml output
    if a.xml_particle_document:
        out_opt["full_document"] = False
    if a.xml_disable_pretty:
        out_opt["pretty"] = False
    return in_opt, out_opt


def __int(x):
    return int(float(x))


def main() -> None:
    """
    Enterypoint for CLI command 'fconv'
    """
    p = _set_opts()
    llevel = DEBUG if p.debug else ERROR
    Logger(llevel)

    src_fmt = p.source
    target_fmt = p.target
    src_path = p.i
    target_path = p.o

    in_opt, out_opt = parse_args(p)

    r = Former(
        src_format=SUPPORTED_FORMATS.get(src_fmt),
        target_format=SUPPORTED_FORMATS.get(target_fmt),
        src_path=src_path,
        target_path=target_path,
        in_opt=in_opt,
        out_opt=out_opt,
    ).form()

    if not target_path:
        print(r)


if __name__ == "__main__":
    main()



import typing as t
from typing import Any, Dict

from fconv.formats import get_supported_formats
from fconv.formats.base import BaseFormat
from fconv.util import Logger, logg


class Former:
    """Object for converting input file or string into other formatted file or string.
    Keyword Arguments:
        - `src_format` (Required)       :The format name of the file converting from.
        - `target_format` (Required)    :The format name of the file converting to.
        - `src_path`                    :The name of the file converting from.
        - `target_path`                 :The name of the file converting to.
        - `in_opt`                      :The option sets of file format converting from.
        - `out_opt`                     :The option sets of file format converting to.

        To know which options are available,
        See the reference:
            - json -- https://docs.python.org/3/library/json.html
            - yaml -- https://pyyaml.org/wiki/PyYAMLDocumentation

    Example:
        >>> from fconv.core import Former
        >>> from fconv.formats.json import Json
        >>> from fconv.formats.yaml import Yaml
        >>> f = Former(src_format=Json, target_format=Yaml)

        >>> f.form('{"key1":"value1"}')
        'key1: value1\\n'

        >>> f = Former(src_format=Yaml, target_format=Json, out_opt={'indent':3})
        >>> f.form('key1: value1\\n')
        '{\\n   "key1": "value1"\\n}'

    """

    __slots__ = [
        "_src_format",
        "_target_format",
        "src_path",
        "target_path",
        "in_opt",
        "out_opt",
    ]

    @logg
    def __init__(
        self,
        src_format,
        target_format,
        src_path: t.Optional[str] = "",
        target_path: t.Optional[str] = "",
        in_opt: t.Optional[Dict[str, Any]] = None,
        out_opt: t.Optional[Dict[str, Any]] = None,
    ):

        self._src_format = src_format
        self._target_format = target_format

        if not self.is_valid_format:
            raise ValueError(f"Invalid format. expect: {get_supported_formats()}")

        self.src_path = src_path
        self.target_path = target_path
        self.in_opt = in_opt
        self.out_opt = out_opt

    def __repr__(self):
        return f"{self.__class__.__module__}"

    @property
    def is_valid_format(self) -> bool:
        return issubclass(self._src_format, BaseFormat) and issubclass(
            self._target_format, BaseFormat
        )

    @logg
    def form(self, src_ctx: str = "") -> str:
        log = Logger().logger
        log.debug(f"{self=}")
        if self.src_path and not src_ctx:
            src_ctx = self._read_file(self.src_path)

        target_ctx = self._parse_from_internal(
            self._parse_to_internal(src_ctx, self.in_opt), self.out_opt
        )

        if self.target_path:
            self._write_file(target_ctx, self.target_path)

        return target_ctx

    def _parse_to_internal(self, ctx: str, opt: t.Optional[Dict[str, Any]]) -> Any:
        """
        1. Combine  ctx(raw input) and option passed by client.
        2. Convert them into dictionary.
        """
        src_format = self._src_format()
        return src_format.load(src_format.get_load_kwargs(ctx, opt))

    def _parse_from_internal(
        self, internal: Any, opt: t.Optional[Dict[str, Any]]
    ) -> str:
        """
        1. Combine  intrernal(dictionary converted by load function) and
        option passed by client.
        2. Convert them into string.
        """
        target_format = self._target_format()
        return target_format.dump(target_format.get_dump_kwargs(internal, opt))

    @staticmethod
    def _read_file(fname: str) -> str:
        with open(fname) as f:
            return f.read()

    @staticmethod
    def _write_file(ctx: str, fname: str) -> None:
        f = open(fname, "x+")
        f.write(ctx)


if __name__ == "__main__":
    import doctest

    doctest.testmod(verbose=True)



__doc__ = "Converter between multiple open-standard file formats."
__version__ = "3.2.2"
__prog__ = "fconv"
__author__ = "wf001"
__license__ = "MIT"
__url__ = "https://github.com/wf001/fconv"

HELP = {
    "source": "data format converting from",
    "infile": "(Required) a file converting from",
    "target": "data format converting to",
    "outfile": "a file converting to",
    "in_opt": "option sets of a file converting from",
    "out_opt": "option sets of a file converting to",
    "version": "print version number and exit",
    "debug": "print more information",
    # json input
    "json_float_as_str": "JSON float to be decoded with string",
    "json_float_as_int": "JSON float to be decoded with int",
    "json_int_as_str": "JSON int to be decoded with string",
    "json_int_as_float": "JSON int to be decoded with float",
    # json output
    "json_skip_keys": " keys that are not of a basic type (str, int, float, bool, None) will be skipped",
    "json_ignore_check_circular": "the circular reference check for container types will be skipped",
    "json_disallow_nan": "disallow to serialize out of range float values (nan, inf, -inf) in strict compliance of the JSON specification. ",
    "json_indent": "JSON array elements and object members will be pretty-printed with that indent level. ",
    "json_sort_keys": "the output of dictionaries will be sorted by key.",
    # yaml output
    "yaml_explicit_start": "add explicit start marker(See also https://yaml.org/spec/1.2.2/#914-explicit-documents)",
    "yaml_explicit_end": "add explicit end marker(See also https://yaml.org/spec/1.2.2/#914-explicit-documents)",
    "yaml_indent": "YAML array elements and object members will be pretty-printed with that indent level. ",
    # xml input
    "xml_process_namespaces": "enable namespace support(See more info at https://github.com/martinblech/xmltodict#namespace-support)",
    "xml_process_comments": "treat comments directives as an attribute",
    # xml output
    "xml_particle_document": "disable to add document root attribute",
    "xml_disable_pretty": "disable XML document to be pretty-printed with indent",
}



from typing import Any, Dict

from .base import BaseDictionalizeFormat


class Toml(BaseDictionalizeFormat):
    def __init__(self):
        super().__init__(load_data_key="s", dump_data_key="o")

    @staticmethod
    def load(src_ctx: Dict[str, Any]) -> Dict[str, Any]:
        import toml

        return toml.loads(**src_ctx)

    @staticmethod
    def dump(internal: Dict[str, Any]) -> str:
        import toml

        return toml.dumps(**internal)





from setuptools import setup, find_packages
import fconv


def _requires_from_file(filename):
    return open(filename).read().splitlines()


def long_description():
    with open('README.md') as f:
        return f.read()


setup(
    name='fconv',
    version=fconv.__version__,
    description=fconv.__doc__,
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author=fconv.__author__,
    url=fconv.__url__,
    author_email="wf001@diax.xyz",
    license=fconv.__license__,
    entry_points={
        'console_scripts': [
            'fconv = fconv.__main__:main',
        ],
    },
    packages=find_packages(include=['fconv', 'fconv.*']),
    install_requires=_requires_from_file('requirement.txt'),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing"
        ]
)



from typing import Any, Dict

from .base import BaseDictionalizeFormat


class Yaml(BaseDictionalizeFormat):
    def __init__(self):
        super().__init__(load_data_key="stream", dump_data_key="data")

    @staticmethod
    def load(src_ctx: Dict[str, Any]) -> Dict[str, Any]:
        import yaml

        return yaml.safe_load(**src_ctx)

    @staticmethod
    def dump(internal: Dict[str, Any]) -> str:
        import yaml

        return yaml.dump(**internal)



from typing import Any, Dict

from .base import BaseDictionalizeFormat


class Xml(BaseDictionalizeFormat):
    def __init__(self):
        super().__init__(load_data_key="xml_input", dump_data_key="input_dict")

    @staticmethod
    def load(src_ctx: Dict[str, Any]) -> Dict[str, Any]:
        import xmltodict as xml

        return xml.parse(**src_ctx)

    @staticmethod
    def dump(internal: Dict[str, Any]) -> str:
        import xmltodict as xml

        _wrapped = {}
        _internal = internal

        _body = internal["input_dict"]
        _wrapped["root"] = _body
        _internal["input_dict"] = _wrapped
        _internal["pretty"] = True if "pretty" not in _internal else _internal["pretty"]

        return xml.unparse(**_internal)
